% Initializes connection to local buffers of the DataRiver system or sets up variables for simulated real-time.

global mr
% field description
% accumulatedData;                    holds accumulated data received from the buffer
% numberOfChannelsInAccumulatedData;  hold the number of channels that are valid (have received data) in accumulatedData variable
% numberOfFramesInAccumulatedData;    number of valid frames accumulatedData variable
% numberOfFramesReceived               number of data samples (frames) received during the current session.
%                                     This is similar but different from numberOfFramesInAccumulatedData,
%                                     becuse if maxNumberOfFramesInAccumulatedData is set no a value other than Inf,
%                                     accumulated data will be shrunk at some point, which changes the value in numberOfFramesInAccumulatedData,
%                                     but leaves numberOfFrameReceived unchanged.
% bufferHandle;                       global handle to DataRiver buffer opened with ds_OpenAsyncRead
% functionHandle;                     global function (handle) to tun after data is accumulated

% verboseLevel;
% readFromDataRiver                   indicates whether to read from DataRiver or from a variable, simulating realtime


% set up (add to path) all the necessary directories (relative to mr_init() path)
selfDirectory = fileparts(which('mr_init'));
addpath([selfDirectory '/']);
addpath([selfDirectory '/SDK/']);
addpath([selfDirectory '/SDK/m/']);
addpath([selfDirectory '/SDK/m/t/']);
addpath([selfDirectory '/circular_buffer/']);

maxChannels = 100;
initialLength = 10000;
mr.verboseLevel = 1; % 0 is no verbosity, 1 for medium and 2 is for max

mr.accumulatedData = nan(maxChannels, initialLength);
mr.numberOfChannelsInAccumulatedData = nan;
mr.numberOfFramesInAccumulatedData = 0;

mr.numberOfFramesReceived = 0;

mr.event = [];

% when data is received from DataRiver, the first channel is always
% the event channel. If data is simulated (read in EEGLAB from a bdf file)
% the event channel is always the last channel
if mr.readFromDataRiver
    mr.eventChannelNumber = 1;
    mr.eeg.channelOffset = 1;
else
    % last channel of simulated data should contain the event channel,
    % similar to the way bdf files are imported.
    
    % give an error is not data is available for the simulated mode. The
    % dimension of this data is used to set the event channel number.
    if ~isfield(mr, 'simulation') || ~isfield(mr.simulation, 'data') || isempty(mr.simulation.data)
        error('MatRiver: The field mr.simulation.data should exist and contain data before running mr_init()\n');
    end;
    
    
    mr.eventChannelNumber = size(mr.simulation.data, 1);
    
    % no eeg channel offset in the simulated mode.
    mr.eeg.channelOffset = 0;
end;

mr.eeg.accumulatedIcActivity = []; % for now, as we assume that we just concatenate new data and grow this, should change soon as it makes thing slow
mr.eeg.badChannels = [];

% for ica
if isfield(mr.eeg, 'originalIcaUnmix')
    mr.eeg.activeIcaUnmix = mr.eeg.originalIcaUnmix; % active ica unmix is used to ignore bad chanels
end;


if mr.readFromDataRiver
    %     % open Data suite (ds)library if it has not been opened before
    %     addpath 'C:\tools\mr\SDK\m';
    %     addpath 'C:\tools\mr\SDK\';
    
    if ~exist('ds_lib')
        startup_ds;
        
        if mr.verboseLevel > 0
            fprintf('Data Suite bridge initiated.\n');
        end;
    end;
    
     
    bufferName = '/tmp/DataRiver';
    mr.readBufferHandle = ds_OpenRead(bufferName);
    if mr.readBufferHandle == 0
        fprintf('%s buffer could not be opened.\n', bufferName);
        keyboard;
    else
        fprintf('Reading from %s buffer..\n', bufferName);
    end;
    
    if mr.verboseLevel > 0
        fprintf('DataRiver buffer handle acquired.\n');
    else
        fprintf('Simulated Data mode started.\n');
    end
end;